@@ -42,6 +42,7 @@ ConnectionIcon::ConnectionIcon(QObject* parent)
     , m_connecting(false)
     , m_limited(false)
     , m_vpn(false)
+    , m_airplaneMode(false)
 #if WITH_MODEMMANAGER_SUPPORT
     , m_modemNetwork(0)
 #endif
@@ -109,6 +110,21 @@ QString ConnectionIcon::connectionTooltipIcon() const
     return m_connectionTooltipIcon;
 }
 
+bool ConnectionIcon::airplaneMode() const
+{
+    return m_airplaneMode;
+}
+
+void ConnectionIcon::setAirplaneMode(bool airplaneMode)
+{
+    if (m_airplaneMode != airplaneMode) {
+        m_airplaneMode = airplaneMode;
+        Q_EMIT airplaneModeChanged(airplaneMode);
+
+        setIcons();
+    }
+}
+
 void ConnectionIcon::activatingConnectionChanged(const QString& connection)
 {
     Q_UNUSED(connection);
@@ -444,6 +460,11 @@ void ConnectionIcon::setIcons()
 
 void ConnectionIcon::setDisconnectedIcon()
 {
+    if (m_airplaneMode) {
+        setConnectionIcon(QStringLiteral("network-flightmode-on"));
+        return;
+    }
+
     if (NetworkManager::status() == NetworkManager::Unknown ||
         NetworkManager::status() == NetworkManager::Asleep) {
         setConnectionIcon("network-unavailable");
